mod markdown;

use std::{
    ffi::OsString,
    path::{Path, PathBuf},
};

use walkdir::WalkDir;

fn main() {
    let src_dir = {
        let def = || OsString::from("./content/");
        let var = std::env::var_os("SOURCE_DIR").unwrap_or_else(def);
        PathBuf::from(var)
    };

    let dst_dir = {
        let def = || OsString::from("./dist/");
        let var = std::env::var_os("DIST_DIR").unwrap_or_else(def);
        PathBuf::from(var)
    };

    for ent in WalkDir::new(&src_dir) {
        let Ok(ent) = ent else { continue };

        let src = ent.path();

        if !src.is_file() && !src.is_symlink() {
            continue;
        }

        match src.extension().map(|ext| ext.as_encoded_bytes()) {
            Some(b"md") => {
                let (html, gmi) = {
                    let dst = get_dist_path(src, &src_dir, &dst_dir);
                    (dst.with_extension("html"), dst.with_extension("gmi"))
                };
                if check_outdated(src, &html) || check_outdated(src, &gmi) {
                    let _ = try_mkdirs(&html);
                    if let Err(e) = markdown::compile_md_file(src, html, gmi) {
                        eprintln!("Could not compile content: {:?}", e);
                    }
                }
            }

            Some(b"css") => {
                let dst = get_dist_path(src, &src_dir, &dst_dir);
                if check_outdated(src, &dst) {
                    let _ = try_mkdirs(&dst);
                    //
                }
            }

            _ => {}
        }
    }
}

#[inline]
fn get_dist_path(
    src: impl AsRef<Path>,
    src_dir: impl AsRef<Path>,
    dst_dir: impl AsRef<Path>,
) -> PathBuf {
    dst_dir
        .as_ref()
        .join(src.as_ref().strip_prefix(&src_dir).unwrap())
}

#[inline]
fn try_mkdirs(dst: impl AsRef<Path>) -> std::io::Result<()> {
    std::fs::create_dir_all(dst.as_ref().parent().unwrap_or_else(|| Path::new("/")))
}

fn check_outdated(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> bool {
    let up_to_date = dst
        .as_ref()
        .metadata()
        .and_then(|meta| meta.modified())
        .is_ok_and(|dst_mtime| {
            src.as_ref()
                .metadata()
                .and_then(|meta| meta.modified())
                .is_ok_and(|src_mtime| src_mtime <= dst_mtime)
        });
    // either we cannot get metadata or src_mtime > dst_mtime (outdated)
    !up_to_date
}
